@@ -79,6 +79,7 @@ public:
     String         m_space_type;
     String         m_ten_key_type;
     String         m_behavior_on_period;
+    String         m_behavior_on_focus_out;
 
     bool           m_show_candidates_label;
     bool           m_close_cand_win_on_select;
@@ -91,10 +92,15 @@ public:
     bool           m_romaji_half_symbol;
     bool           m_romaji_half_number;
     bool           m_romaji_allow_split;
+
+    bool           m_romaji_pseudo_ascii_mode;
+    bool           m_romaji_pseudo_ascii_blank_behavior;
+
     int            m_nicola_time;
     KeyEventList   m_left_thumb_keys;
     KeyEventList   m_right_thumb_keys;
 
+    String         m_dict_encoding;
     String         m_dict_admin_command;
     String         m_add_word_command;
 
