@@ -194,9 +194,25 @@ AnthyInstance::process_key_event_input (const KeyEvent &key)
 bool
 AnthyInstance::process_key_event_lookup_keybind (const KeyEvent& key)
 {
+    std::vector<Action>::iterator it;
+
     m_last_key = key;
 
-    std::vector<Action>::iterator it;
+    /* try to find a "insert a blank" action to be not stolen a blank key
+     * when entering the pseudo ascii mode.
+     */
+    if (get_pseudo_ascii_mode () != 0 &&
+        m_factory->m_romaji_pseudo_ascii_blank_behavior &&
+        m_preedit.is_pseudo_ascii_mode ()) {
+        for (it  = m_factory->m_actions.begin();
+             it != m_factory->m_actions.end();
+             it++) {
+            if (it->match_action_name ("INSERT_SPACE") &&
+                it->perform (this, key)) {
+                return true;
+            }
+        }
+    }
     for (it  = m_factory->m_actions.begin();
          it != m_factory->m_actions.end();
          it++)
@@ -314,7 +330,8 @@ AnthyInstance::select_candidate_no_direct (unsigned int item)
     if (m_preedit.is_predicting () && !m_preedit.is_converting ())
         action_predict ();
 
-    if (!is_selecting_candidates ()) return;
+    if (!is_selecting_candidates ())
+        return;
 
     // update lookup table
     m_lookup_table.set_cursor_pos_in_current_page (item);
@@ -439,6 +456,15 @@ AnthyInstance::focus_out ()
 {
     SCIM_DEBUG_IMENGINE(2) << "focus_out.\n";
 
+    if (m_preedit.is_preediting ()) {
+        if (m_factory->m_behavior_on_focus_out == "Clear")
+            reset ();
+        else if (m_factory->m_behavior_on_focus_out == "Commit")
+            action_commit (m_factory->m_learn_on_auto_commit);
+        else
+            action_commit (m_factory->m_learn_on_auto_commit);
+    }
+
     Transaction send;
     send.put_command (SCIM_TRANS_CMD_REQUEST);
     send.put_command (SCIM_TRANS_CMD_FOCUS_OUT);
@@ -819,11 +845,14 @@ AnthyInstance::set_typing_method (TypingMethod method)
 
     if (method != get_typing_method ()) {
         Key2KanaTable *fundamental_table = NULL;
-        if (method == SCIM_ANTHY_TYPING_METHOD_ROMAJI)
+
+        if (method == SCIM_ANTHY_TYPING_METHOD_ROMAJI) {
             fundamental_table = m_factory->m_custom_romaji_table;
-        else if (method == SCIM_ANTHY_TYPING_METHOD_KANA)
+        } else if (method == SCIM_ANTHY_TYPING_METHOD_KANA) {
             fundamental_table = m_factory->m_custom_kana_table;
+        }
         m_preedit.set_typing_method (method);
+        m_preedit.set_pseudo_ascii_mode (get_pseudo_ascii_mode ());
     }
 }
 
@@ -1110,15 +1139,17 @@ AnthyInstance::action_delete (void)
 bool
 AnthyInstance::action_insert_space (void)
 {
-    if (m_preedit.is_preediting ())
-        return false;
+    String str;
+    bool is_wide = false, retval = false;
 
-    bool is_wide = false;
+    if (m_preedit.is_preediting () && !m_factory->m_romaji_pseudo_ascii_blank_behavior)
+        return false;
 
     if (m_factory->m_space_type == "FollowMode") {
         InputMode mode = get_input_mode ();
         if (mode == SCIM_ANTHY_MODE_LATIN ||
-            mode == SCIM_ANTHY_MODE_HALF_KATAKANA)
+            mode == SCIM_ANTHY_MODE_HALF_KATAKANA ||
+            m_preedit.is_pseudo_ascii_mode ())
         {
             is_wide = false;
         } else {
@@ -1129,27 +1160,39 @@ AnthyInstance::action_insert_space (void)
     }
 
     if (is_wide) {
-        commit_string (utf8_mbstowcs ("\xE3\x80\x80"));
-        return true;
+        str = "\xE3\x80\x80";
+        retval = true;
     } else if (get_typing_method () == SCIM_ANTHY_TYPING_METHOD_NICOLA || // FIXME! it's a ad-hoc solution.
+               m_preedit.is_pseudo_ascii_mode () ||
                (m_last_key.code != SCIM_KEY_space &&
                 m_last_key.code != SCIM_KEY_KP_Space))
     {
-        commit_string (utf8_mbstowcs (" "));
-        return true;
+        str = " ";
+        retval = true;
     }
 
-    return false;
+    if (retval) {
+        if (m_preedit.is_pseudo_ascii_mode ()) {
+            m_preedit.append (m_last_key, str);
+            show_preedit_string ();
+            m_preedit_string_visible = true;
+            set_preedition ();
+        } else {
+            commit_string (utf8_mbstowcs (str));
+        }
+    }
+
+    return retval;
 }
 
 bool 
 AnthyInstance::action_insert_alternative_space (void)
 {
+    bool is_wide = false;
+
     if (m_preedit.is_preediting ())
         return false;
 
-    bool is_wide = false;
-
     if (m_factory->m_space_type == "FollowMode") {
         InputMode mode = get_input_mode ();
         if (mode == SCIM_ANTHY_MODE_LATIN ||
@@ -1381,6 +1424,8 @@ AnthyInstance::action_commit_first_segment (void)
     commit_string (m_preedit.get_segment_string (0));
     if (m_factory->m_learn_on_manual_commit)
         m_preedit.commit (0);
+    else
+        m_preedit.clear (0);
 
     set_preedition ();
 
@@ -1404,6 +1449,8 @@ AnthyInstance::action_commit_selected_segment (void)
         commit_string (m_preedit.get_segment_string (i));
     if (m_factory->m_learn_on_manual_commit)
         m_preedit.commit (m_preedit.get_selected_segment ());
+    else
+        m_preedit.clear (m_preedit.get_selected_segment ());
 
     set_preedition ();
 
@@ -1426,6 +1473,8 @@ AnthyInstance::action_commit_first_segment_reverse_preference (void)
     commit_string (m_preedit.get_segment_string (0));
     if (!m_factory->m_learn_on_manual_commit)
         m_preedit.commit (0);
+    else
+        m_preedit.clear (0);
 
     set_preedition ();
 
@@ -1449,6 +1498,8 @@ AnthyInstance::action_commit_selected_segment_reverse_preference (void)
         commit_string (m_preedit.get_segment_string (i));
     if (!m_factory->m_learn_on_manual_commit)
         m_preedit.commit (m_preedit.get_selected_segment ());
+    else
+        m_preedit.clear (m_preedit.get_selected_segment ());
 
     set_preedition ();
 
@@ -1559,7 +1610,9 @@ AnthyInstance::action_candidates_page_down (void)
 bool
 AnthyInstance::action_select_candidate (unsigned int i)
 {
-    if (!m_lookup_table_visible) return false;
+    // FIXME! m_lookup_table_visible should be set as true also on predicting
+    if (!m_lookup_table_visible && !m_preedit.is_predicting ())
+        return false;
 
     if (m_preedit.is_predicting () && !m_preedit.is_converting () &&
         m_factory->m_use_direct_key_on_predict)
@@ -1769,6 +1822,19 @@ AnthyInstance::action_half_katakana_mode (void)
     return true;
 }
 
+bool
+AnthyInstance::action_cancel_pseudo_ascii_mode (void)
+{
+    if (!m_preedit.is_preediting ())
+        return false;
+    if (!m_preedit.is_pseudo_ascii_mode ())
+        return false;
+
+    m_preedit.reset_pseudo_ascii_mode ();
+
+    return true;
+}
+
 bool
 AnthyInstance::convert_kana (CandidateType type)
 {
@@ -1994,7 +2060,7 @@ AnthyInstance::timeout_add (uint32 time_msec, timeout_func timeout_fn,
     /*
      * FIXME! Obsoleted closures should be removed at somewhere.
      * Currenly only NICOLA related timer uses this feature and it will be
-     * removed each time on key press event so memory leaks doesn't exit.
+     * removed each time on key press event so memory leaks doesn't exist.
      */
 
     Transaction send;
@@ -2110,6 +2176,8 @@ AnthyInstance::process_helper_event (const String &helper_uuid,
     switch (cmd) {
     case SCIM_ANTHY_TRANS_CMD_GET_SELECTION:
     {
+        // For reconversion feature, but this code is ad-hoc solution.
+
         WideString selection, surround;
         if (!reader.get_data (selection) || selection.empty ())
             break;
@@ -2117,22 +2185,31 @@ AnthyInstance::process_helper_event (const String &helper_uuid,
         int cursor;
         unsigned int len = selection.length ();
         if (!get_surrounding_text (surround, cursor, len, len))
-            break;
-
-        if (surround.length () - cursor >= len &&
-            surround.substr (cursor, len) == selection)
         {
-            delete_surrounding_text (0, len);
-            m_preedit.convert (selection);
-            set_preedition ();
-            set_lookup_table ();
-        } else if (cursor >= (int) len &&
-                   surround.substr (cursor - len, len) == selection)
+            // We expect application to delete selection text.
+            m_preedit.convert(selection);
+            set_preedition();
+            set_lookup_table();
+        }
+        else
         {
-            delete_surrounding_text (0 - len, len);
-            m_preedit.convert (selection);
-            set_preedition ();
-            set_lookup_table ();
+            // This code will conflict if same string exists at both before and
+            // after the caret.
+            if (surround.length () - cursor >= len &&
+                surround.substr (cursor, len) == selection)
+            {
+                delete_surrounding_text (0, len);
+                m_preedit.convert (selection);
+                set_preedition ();
+                set_lookup_table ();
+            } else if (cursor >= (int) len &&
+                       surround.substr (cursor - len, len) == selection)
+            {
+                delete_surrounding_text (0 - len, len);
+                m_preedit.convert (selection);
+                set_preedition ();
+                set_lookup_table ();
+            }
         }
         break;
     }
@@ -2173,16 +2250,19 @@ AnthyInstance::reload_config (const ConfigPointer &config)
             m_preedit.set_input_mode (SCIM_ANTHY_MODE_WIDE_LATIN);
     }
 
-    // set typing method
+    // set typing method and pseudo ASCII mode
     if (m_on_init || !m_factory->m_show_typing_method_label) {
-        if (m_factory->m_typing_method == "NICOLA")
+        if (m_factory->m_typing_method == "NICOLA") {
             m_preedit.set_typing_method (SCIM_ANTHY_TYPING_METHOD_NICOLA);
-        else if (m_factory->m_typing_method == "Kana")
+        } else if (m_factory->m_typing_method == "Kana") {
             m_preedit.set_typing_method (SCIM_ANTHY_TYPING_METHOD_KANA);
-        else
+        } else {
             m_preedit.set_typing_method (SCIM_ANTHY_TYPING_METHOD_ROMAJI);
+        }
+        m_preedit.set_pseudo_ascii_mode (get_pseudo_ascii_mode ());
     } else {
-        m_preedit.set_typing_method (m_preedit.get_typing_method ());
+        m_preedit.set_typing_method (get_typing_method ());
+        m_preedit.set_pseudo_ascii_mode (get_pseudo_ascii_mode ());
     }
 
     // set conversion mode
@@ -2246,6 +2326,9 @@ AnthyInstance::reload_config (const ConfigPointer &config)
     // setup toolbar
     m_properties.clear ();
     install_properties ();
+
+    // set encoding
+    m_preedit.set_dict_encoding (m_factory->m_dict_encoding);
 }
 
 bool
@@ -2267,6 +2350,21 @@ AnthyInstance::is_realtime_conversion (void)
     else
         return false;
 }
+
+int
+AnthyInstance::get_pseudo_ascii_mode (void)
+{
+    int retval = 0;
+    TypingMethod m = get_typing_method ();
+
+    if (m == SCIM_ANTHY_TYPING_METHOD_ROMAJI) {
+        if (m_factory->m_romaji_pseudo_ascii_mode)
+            retval |= SCIM_ANTHY_PSEUDO_ASCII_TRIGGERED_CAPITALIZED;
+    }
+
+    return retval;
+}
+
 /*
 vi:ts=4:nowrap:ai:expandtab
 */
