@@ -27,10 +27,12 @@ using namespace scim_anthy;
 
 Key2KanaConvertor::Key2KanaConvertor (AnthyInstance    & anthy,
                                       Key2KanaTableSet & tables)
-    : m_anthy          (anthy),
-      m_tables         (tables)
+    : m_anthy                   (anthy),
+      m_tables                  (tables),
+      m_is_in_pseudo_ascii_mode (false)
 {
     set_case_sensitive (false);
+    set_pseudo_ascii_mode (0);
 }
 
 Key2KanaConvertor::~Key2KanaConvertor ()
@@ -38,7 +40,8 @@ Key2KanaConvertor::~Key2KanaConvertor ()
 }
 
 bool
-Key2KanaConvertor::can_append (const KeyEvent & key)
+Key2KanaConvertor::can_append (const KeyEvent & key,
+                               bool             ignore_space)
 {
     // ignore key release.
     if (key.is_key_release ())
@@ -51,7 +54,8 @@ Key2KanaConvertor::can_append (const KeyEvent & key)
         return false;
     }
 
-    if (isprint(key.get_ascii_code ()) && !isspace(key.get_ascii_code ()))
+    if (isprint(key.get_ascii_code ()) &&
+        (ignore_space || !isspace(key.get_ascii_code ())))
         return true;
 
     if (util_key_is_keypad (key))
@@ -124,6 +128,11 @@ Key2KanaConvertor::append (const String & str,
     bool has_partial_match = false;
     bool retval = false;
 
+    if (m_pseudo_ascii_mode != 0 && process_pseudo_ascii_mode (widestr)) {
+        m_pending += widestr;
+        pending = m_pending;
+        return false;
+    }
     if (!m_case_sensitive) {
         String half = utf8_wcstombs (matching_str);
         for (unsigned int i = 0; i < half.length (); i++)
@@ -213,6 +222,7 @@ Key2KanaConvertor::clear (void)
 {
     m_pending.clear ();
     m_exact_match.clear ();
+    reset_pseudo_ascii_mode();
 }
 
 bool
@@ -257,6 +267,31 @@ Key2KanaConvertor::reset_pending (const WideString &result, const String &raw)
         append (raw.substr(i, 1), res, pend);
     }
 }
+
+bool
+Key2KanaConvertor::process_pseudo_ascii_mode (const WideString & wstr)
+{
+    for (unsigned int i = 0; i < wstr.length (); i++) {
+        if ((wstr[i] >= 'A' && wstr[i] <= 'Z') ||
+            iswspace(wstr[i]))
+        {
+            m_is_in_pseudo_ascii_mode = true;
+        } else if (wstr[i] >= 0x80) {
+            m_is_in_pseudo_ascii_mode = false;
+        }
+    }
+
+    return m_is_in_pseudo_ascii_mode;
+}
+
+void
+Key2KanaConvertor::reset_pseudo_ascii_mode (void)
+{
+    if (m_is_in_pseudo_ascii_mode)
+        m_pending.clear();
+    m_is_in_pseudo_ascii_mode = false;
+}
+
 /*
 vi:ts=4:nowrap:ai:expandtab
 */
