@@ -186,7 +186,7 @@
 #define SCIM_ANTHY_CONFIG_LEARN_ON_MANUAL_COMMIT_DEFAULT      true
 #define SCIM_ANTHY_CONFIG_LEARN_ON_AUTO_COMMIT_DEFAULT        true
 
-#define SCIM_ANTHY_CONFIG_DICT_ENCODING_DEFAULT               "EUC-JP-MS"
+#define SCIM_ANTHY_CONFIG_DICT_ENCODING_DEFAULT               "EUC-JP"
 #define SCIM_ANTHY_CONFIG_DICT_ADMIN_COMMAND_DEFAULT          "kasumi"
 #define SCIM_ANTHY_CONFIG_ADD_WORD_COMMAND_DEFAULT            "kasumi --add"
 
